'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'' DART BOARD Controller Driver and Utility 
'  http://www.boardbuilders.co
'  
'  For use with the Dart Board RCM.
'  Temperature inputs on pin 1-4
'  Pressure inputs on pin 5,6
'  Relays control via pin 18,19,20
'  FEATURES
'   connect sensors to relay independantly via software
'   set different on and off trigger levels for maintaining readings within a range
'   cofnigur triggers for rising or falling transitions
'   set relay trigger based on difference between 2 sensors 
'   simultaneously plot on 2 charts with independant time scales (quick and slow)
'   optional chart labels
'   savable triggers and chart configurations via menus

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

'--------------- Operating Parameters ----------------------------------------

' System operating parameters

v_supply =4.95                ' supply voltage to pressure sensor
temp_limit = 90               ' maximum temperature for thermocouple amps
press_limit = 30              ' maximum sensor pressure


' User definable parameters
chart_config_file$="Cdef.txt" ' file name of default plot settings
trig_file_name$="TrDef.txt"  ' default trigger file name
p_tolerance = 0.2             '(psi) calibration tolerance against 1 atmosphere

'----------------Data Structures ---------------------------------------
'TODO implement savable temperate profiles
Dim Temp1_profile_temperatures(20)
Dim Temp1_profile_durations(20)

'----------------Relays Object ----------------------------------------

' properties 
    Dim relay_status$(3)

' Methods
' Initialisers
    Sub Init_Relays
    ' set pins 11,12,13 as digital outputs to drive the relays
        Print "Iniializing relays"
            SetPin 18,8  ' relay 3
            SetPin 19,8  ' relay 1
            SetPin 20,8  ' relay 2
            relay_status$(1)="off"
            relay_status$(2)="off"
            relay_status$(3)="off"
        End Sub

' Test Relays
    Sub Test_Relays
          Print "Testing Relay Activity: Relays cylcing on/off once each"
        Print "... cycling relay 1"
          close_relay 1
          Pause 400
          open_relay 1
          Pause 400
        Print "... cycling relay 2"
          close_relay 2
          Pause 400
          open_relay 2
          Pause 400
        Print "... cycling relay 3"
          close_relay 3
          Pause 400
          open_relay 3
    End Sub

'CLOSE  the relay contacts ( turn ON)
    Sub close_relay(relayNumber)
        If relayNumber=3 Then
            Pin(18)=1
            relay_status$(3)="on"
        EndIf

        If relayNumber=1 Then
            Pin(19)=1
            relay_status$(1)="on"
        EndIf

        If relayNumber=2 Then
            Pin(20)=1
            relay_status$(2)="on"
        EndIf
    End Sub

' OPEN the relay contacts ( turn OFF)
    Sub open_relay(relayNumber)
        If relayNumber = 3 Then
            Pin(18)=0
            relay_status$(3)="off"
        EndIf

        If relayNumber=1 Then
            Pin(19)=0
            relay_status$(1)="off"
        EndIf

        If relayNumber=2 Then
            Pin(20)=0
            relay_status$(2)="off"
        EndIf
    End Sub

'--------------------Temperature Sensor Object------------------------------

' Methods
' INIT sensors
    Sub Init_thermocouples
           ' set the transducer input lines to be analoge inputs
        Print "Initialising Thermcouples"
        SetPin 4,1  'temp sensor 4
        SetPin 3,1  'temp sensor 3
        SetPin 2,1  'temp sensor 2
        SetPin 1,1  'temp sensor 1
    End Sub

' TEST scan of thermocouples (note when thermo couple removed input goes high)
    Sub test_thermocouples
          Local blConnected, test_temp1,test_temp2,test_temp3,test_temp4
          blConnected=0

          ' read the thermocouple values
          read_temp 1, test_temp1
          read_temp 2, test_temp2
          read_temp 3, test_temp3
          read_temp 4, test_temp4
          'Print test_temp1,test_temp2,test_temp3,test_temp4
          Print "Testing Thermocouples"
            '' if thermocouple not connected reading maxes out
          If test_temp1>temp_limit*0.9 Then
              Print "     Sensor 1: N.C"
            Else
              Print "    Sensor 1:", test_temp1, "C"
              blConnected=1
          EndIf

          If (test_temp2>temp_limit*0.9) Then
              Print "     Sensor 2: N.C"
            Else
              Print "    Sensor 2:", test_temp2, "C"
              blConnected=1
          EndIf

          If (test_temp3>temp_limit*0.9) Then
              Print "     Sensor 3: N.C"
            Else
              Print "    Sensor 3:", test_temp3, "C"
              blConnected =1
          EndIf

          If test_temp4>temp_limit*0.9 Then
              Print "     Sensor 4: N.C"
                Else
            Print "     Senor 4:", Format$(test_temp4,"%.2f"), "C"
              blconnected=1
          EndIf

          If (blConnected=1) Then
            Print " Please confirm is temperatures are correct"
          Else
            Print " Warning: No Thermocouples connected. Please check connections"
          EndIf
    End Sub

' READ temperature sensor
    Sub Read_Temp(sensorNo,temp_reading)
        Local voltage
        If sensorNo=1 Then voltage = Pin(1)
        If sensorNo=2 Then voltage = Pin(2)
        If sensorNo=3 Then voltage = Pin(3)
        If sensorNo=4 Then voltage = Pin(4)
        temp_reading = temperature(voltage)
    End Sub

' UTILITY convert the thermo couple voltage reading to a temperature
    Function temperature(voltage)
        Local conversion_factor
        conversion_factor=0.033 ' units of v/C
        temperature = voltage/conversion_factor
    End Function

' ---------------Presure sensor object -------------------------------------

' METHODS ----
'INIT
    Sub Init_pressure
        Print "Initialise Pressure Sensor"
        SetPin 6,0 ' not used on teh current board
        SetPin 5,1 ' pressure sensor 1
    End Sub

'TEST that the pressure sensor reads close to 1 atmos=14.7 psi ''''
    Sub test_pressure1
      Local press1
      press1=0
      ' get current pressure reading
     Read_Pressure1
      Print "Testing Pressure Sensor 1: ",live_readings(5)," psi"""
      If (press1<=14.7+p_tolerance And live_readings(5)>=14.7-p_tolerance) Then
          Print "    OK: Pressure within 14.7+/-",p_tolerance," psi"
        Else
          Print "    FAILED: Pressure out of tolerance."
          Print "         Pressure should be within 14.7+/-", p_tolerance," psi"
      EndIf
    End Sub

'READ the pressure sensor voltage and convert to pressure
    Sub READ_Pressure1
      Local voltage
      voltage = Pin(5)
      live_readings(5) = pressure(voltage)
        If live_readings(5)<0.95*press_limit Then
        sensor_status$(5)="ok"
      Else
        sensor_Status$(5)="nc"
      EndIf
    End Sub

'UTILTIY convert voltage to pressure reading
    Function pressure(voltage)
      Local factor ' resistor scaling
      factor = 2/6.7 ' resistive divider used for signal conditioning
      pressure = (voltage/factor-0.1*v_supply)*30/(0.8*v_supply)
    End Function


'-------- CONTROLLER PARENT OBJECT -----------------------------------------
'         inherits relay object, thermcouple object, pressure sensor object

' PROPERTIES
    i_period = 250      ' sensor sample cycle in ms

    Dim triggers(3,10)       ' trigger configurations
            ' tigger(,1)    = source sensor ID ( 1-4 is temp, 5 is pressure)
            ' triggers(,2)  =  comparitor for trigger level ( 0<,1>)
            ' triggers (,3) =  ON trigger level 
            ' triggers(,4) = OFF Trigger level
            ' triggers(,5)  = sensor source for comparison (current - this one)
            ' triggers(,6)   = comparitor for compared level
            ' triggers (,7)  = 'difference' trigger level
        'NOTE Absolute triggers and relative value triggers are 'AND'd together 
        '     for determining relay action
    
    Dim alias$(3)            ' relay descriptions
    Dim live_readings(5)    ' reading from 5 sensors 1-4 temp, 5 - pressure
    Dim sensor_status$(5)   ' connected or open circuit ("ok" or "nc")
    
    flgMonitoring = 0       ' 0 = not monitoring, 1-monitoring

    param_count = 10         ' max params per relay. Must equal 2 dim or triggers array.

' METHODS 
'INIT - Configure the pins for the I/Os
    Sub INIT_CONTROLLER
        INIT_Relays
        INIT_pressure
        init_thermocouples
    End Sub

'TEST ALL
    Sub TEST_ALL
    clear_screen
    ' test the relays are responding
      test_relays
    ' test the pressure sensor is within acceptable range
      test_pressure1
    ' Check the thermo couples. Note that if no connection pin will be pulled high
      test_thermocouples
      Print ""
      Input ".... press any key To continue",myk$
    End Sub

' start timed interupt to read sensors
    Sub MONITOR_START
        SetTick i_period,I_ReadSensors
        flgMonitoring = 1
    End Sub

' Stop Timed Interupt
    Sub MONITOR_END
        SetTick 0,I_ReadSensors
        flgMonitoring = 0
    End Sub

' READ All temperature senors into live reading array
    Sub Read_Temp_All
        Local myTemp
        For counter =1 To 4
            Read_temp counter,myTemp
            live_readings(counter)=myTemp
            If myTemp<temp_limit*0.95 Then
                sensor_status$(counter)="ok"
              Else
                sensor_status$(counter)="nc"
              EndIf
         Next counter
    End Sub

' --------- TRIGGER (sub)OBJECT -----------
    Sub CONFIGURE_TRIGGERS
    Local temp$, prompt$
        highlight 0,0,"CONFIGURE TRIGGERS FOR ALL RELAYS",80
        Print " (press enter to keep existing value)"
        Print ""
        Print ""

    For counter = 1 To 3
      Font #1,,1: Print "Relay ";counter: Font #1,1,0
    prompt$="  Source Sensor [0-NONE,1-4 temps; 5 press]:"+Str$(triggers(counter,1))+"  >>"
        Line Input prompt$,temp$
        If Len(temp$)>0 Then triggers(counter,1)=Val(temp$)
    prompt$= "  Alias '"+alias$(counter)+"'   >>"
        Line Input prompt$,temp$
        If Len(temp$)>0 Then alias$(counter)=temp$
    prompt$="  Comparitor for ON (0=Less than,1=greater than):"+Str$(triggers(counter,2))+"    >>"
        Line Input prompt$,temp$
        If Len(temp$)>0 Then triggers(counter,2)=Val(temp$)
    prompt$="  Trigger Level for On:"+Str$(triggers(counter,3))+"    >>"
        Line Input prompt$,temp$
        If Len(temp$)>0 Then triggers(counter,3)=Val(temp$)
    prompt$="  Trigger Level for OFF:"+Str$(triggers(counter,4))+"    >>"
        Line Input prompt$,temp$
        If Len(temp$)>0 Then triggers(counter,4)=Val(temp$)
    prompt$="  Comparison Source [0-None,1-4 temps]:"+Str$(triggers(counter,5))+"     >>"
        Line Input prompt$,temp$
        If Len(temp$)>0 Then triggers(counter,5)=Val(temp$)
        If triggers(counter,5)=0 Then GoTo SkipToNext
    prompt$="  Difference Trigger Level (curr. - comp. source):"+Str$(triggers(counter,6))+"     >>"
       Line Input prompt$,temp$
        If Len(temp$)>0 Then triggers(counter,6)=Val(temp$)
    prompt$="  Comparitor for ON (0=less than, 1=greater than):"+Str$(triggers(counter,7))+"     >>"
       Line Input prompt$,temp$
        If Len(temp$)>0 Then triggers(counter,7)=Val(temp$)
    SkipToNext:
      Next counter
    Print ""
    prompt$ =" TRIGGER FILE NAME (omit extension): "+trig_file_name$+"    >>"
    Line Input prompt$,temp$
    If Len(temp$)>0 Then trig_file_name$=Left$(temp$,6)+".txt"
    ' save changes
      save_triggers trig_file_name$
    End Sub

' SAVE TRIGGERS to disk
    Sub save_Triggers(strName$)
    clear_screen
    prompt$=Space$(20)+"Save current triggers as '"+strName$+"' (y/n)"
    Print @(200,150) ""
    Line Input prompt$,temp$
    If LCase$(Left$(temp$,1))="y" Then
            Open strName$ For output As #1
            For counter =1 To 3
              Print #1,alias$(counter)
                For counter2 =1 To param_count
                    Print #1,triggers(counter,counter2)
                Next counter2
            Next counter
          Close #1
            If MM.Errno<>0 Then
                highlight 1,410,"ERROR SAVING FILE. CODE "+Str$(ERRONO),-1
                Pause 2000
              Else
                highlight 0,410,".... Save OK",-1: Pause 900
            EndIf
    Else
      clear_screen
    EndIf
    End Sub

'LOAD TRIGGERS from file
    Sub load_triggers
    Local myData$, counter,counter2

    clear_screen
    highlight 0,0,"Please choose trigger file or press enter for default",-1
    Print"":Print "Available Files :"
    f$=Dir$("*.txt",file)
    Do While f$<>""
      Print "   "+f$
      f$=Dir$()
    Loop
    Print ""
    prompt$="  Tigger file to load (inc. ext): "+trig_file_name$+"    >>"
    Line Input prompt$,temp$
    If temp$="" Then temp$=trig_file_name$

    Open temp$ For input As #1
    If MM.Errno=0 Then
      For counter =1 To 3
          Line Input #1, alias$(counter)
            For counter2 = 1 To param_count
            Line Input #1,myData$
            triggers(counter,counter2)=Val(myData$)
            Next counter2
        Next counter
        Close #1
        If MM.Errno<>0 Then
            highlight 1,410,"ERROR READING TRIGGER FILE: CODE "+Str$(MM.Errno),-1
            Pause 2000
          Else
          highlight 1,410,"....Triggers Loaded OK",-1
           Pause 800

        EndIf
      Else
        highlight 1,410,"ERROR - Config File Not Found",-1:Pause 2000
      EndIf

    clear_menubar
    End Sub

' EVALUATE AND ACTION A SINGLE TRIGGER
    Sub action_trigger relayNumber
    Local current_reading, falling$, delta_reading, rel_action,abs_action

    rel_action=-1       ' no relative triggers default

    If flgMonitoring=0 Or triggers(relayNumber,1)<=0 Then GoTo not_monitoring

    current_reading = live_readings(triggers(relayNumber,1))

  ' if thermcouple pulled out (input pulled hight) then turn relay off and exit.
    If triggers(relayNumber,1)<=4 Then
        If current_reading>0.95*temp_limit Then
        If relay_status$(relayNumber)="on" Then
         open_relay relayNumber
         GoTo not_monitoring
        Else
         GoTo not_monitoring
        EndIf
        EndIf
     EndIf

  ' indicates if rising (greater than trigger level) or falling edge triggers relay
    If triggers(relayNumber,2)=0 Then falling$="true" Else falling$="false"

  ' Falling edge detection (less than trigger level)
    If falling$="true" Then
        If current_reading<triggers(relayNumber,3) Then
            abs_action = 1             'close_relay relayNumber

        EndIf
    If current_reading>triggers(relayNumber,4) Then
            abs_action = 0              'open_relay relayNumber
         EndIf
    EndIf

    ' rising edge detection
    If falling$="false" Then
     If current_reading>triggers(relayNumber,3) Then
                abs_action = 1          'close_relay relayNumber
            EndIf
     If current_reading<triggers(relayNumber,4) Then
            abs_action=0              'open_relay relayNumber
             EndIf
    EndIf

    ' check relative temperature reading triggers
    ' When configured, the relative condition and absolute condition are
    '   evaluated using AND logic.

    If triggers(relayNumber,5))>0 Then    ' then a comparison trigger is configured
      If live_readings(triggers(relayNumer,5))>0.95*temp_limit Then GoTo action_section
          delta_reading = live_readings(triggers(relayNumber,1))-live_readings(triggers(relayNumber,5))

      If delta_reading>=triggers(relayNumber,6) Then
          If triggers(relayNumber,7)=1 Then
                rel_action=1
              Else
                rel_action=0
          EndIf
      Else
          If triggers(relayNumber,7)=0 Then
                rel_action=1
              Else
                rel_action=0
          EndIf
      EndIf
    EndIf           ' end check comparison trigger

    action_section:
    ' determine whether to open or close relay. Note AND logic used to evaluate action based on absolute and relative measurements
    If rel_action>=0 Then
      If abs_action And rel_action Then
          If relay_status$(relayNumber)="off" Then
            close_relay relayNumber
           EndIf
       Else
          If relay_status$(relayNumber)="on" Then open_relay relayNumber
      EndIf
    Else
      If abs_action =0 And relay_status$(relayNumber)="on" Then open_relay relayNumber
      If abs_action =1 And relay_status$(relayNumber)="off" Then close_relay relayNumber
    EndIf
    not_monitoring: 'jumped here if monitoring is not active or the relayNumber is not connected to a source
    End Sub

' ACTION ALL TRIGGERS
    Sub action_all_triggers
    Local counter
        For counter =1 To 3
            If triggers(counter,1)>0 Then action_trigger counter
        Next counter
    End Sub

'--------- Chart Object ---------------------------------------------------

Dim live_data(120,5)       ' buffer for live data. Operates as circular queue
buffer_size=120           '  Must equal the size of the arrays first dimension

' Misc chart constants
    live_chart_index = 1        ' array index for live data buffer
    reading_count = 0           ' counts number of readings taken adnd resets to zero at LCD value
    
    ' realtime readings params
    monitor_y=40               ' y location of real time monitoring panel
    origin_x = 30              ' lower left corner of chart
    origin_y =MM.VRes -25
    chart_width = MM.HRes -35-origin_x
    chart_height = Cint((MM.VRes-80)/2) ' for 2 char mode
    ' stores curernt x axis value for each chart
    slow_chart_x = origin_x
    quick_chart_x = origin_x

    table_exists$="false"        ' indicates if table outline is currently draw on screen

' Property Defaults Initialisation (over-ridden by chart defaults saved to disk(
    blLogReadings$="false"       ' whether data should be saved to disk
    LogFileName$="Log.txt"          ' Log file name is used

    ' Horizontal axis
    quick_chart_scale = 60   ' number of seconds displayed on quick chart
    slow_chart_scale = 15*60    ' number of seconds on slow chart
    minor_ticks_quick = 10      ' seconds
    major_ticks_quick = 2       ' each minor tick
    minor_ticks_slow = 30       ' seconds
    major_ticks_slow = 2        ' each minor tick

    chart_mode = 3          '  1- quick chart only, 2 - slow chart only , 3-both

    'Vertical Axes
    max_temp = 60               ' degrees C
    max_press = 17             ' psi
    temp_minor_ticks=5         ' seconds
    temp_major_ticks=2          'each tick
    press_minor_ticks=1         ' psi per minor tick
    press_major_ticks=2         ' each minor tick

    ' params for clearing leading area when traces wrap
    refresh_frequency = 20    ' units of readings. x i_period for frequency


Sub CALC_SCREEN_PARAM
    ' screen increments for x andy axes and for fast and slow graphs
    slow_dx = chart_width/slow_chart_scale*i_period/1000
    quick_dx = chart_width/quick_chart_scale*i_period/1000
    temp_dy = (chart_height/max_temp)
    press_dy =(chart_height/max_press)
End Sub

Sub CHART_CONFIG
    highlight 0,0,"CONFIGURE CHARTING OPTIONS",80
    Print "":Print ""
    prompt$=" Chart Mode (1-quick, 2-slow, 3-both): "+Str$(chart_mode)+"  >>"
    Line Input prompt$,temp$
    If Len(temp$)>0 Then chart_mode=Val(temp$)

    prompt$=" maximum temperature(C): "+Str$(max_temp)+"   >>"
    Line Input prompt$,temp$
    If Len(temp$)>0 Then max_temp=Val(temp$)

    prompt$=" maximum pressure(psi): "+Str$(max_press)+"   >>"
    Line Input prompt$,temp$
    If Len(temp$)>0 Then max_press=Val(temp$)

    prompt$=" trace blanking window (readings): "+Str$(refresh_frequency)+"    >>"
    Line Input prompt$,temp$
    If Len(temp$)>0 Then refresh_frequency = Val(temp$)

    highlight -1,-1,"Chart Axes",11

    prompt$=" quick chart x-scale max.(sec): "+Str$(quick_chart_scale)+"    >>"
    Line Input prompt$,temp$
    If Len(temp$)>0 Then quick_chart_scale=Val(temp$)

    prompt$ = " minor time ticks (quick - sec.): "+Str$(minor_ticks_quick)+"    >>"
    Line Input prompt$,temp$
    If Len(temp$)>0 Then minor_ticks_quick=Val(temp$)

    prompt$ = " major time ticks (quick - each minor tick): "+Str$(major_ticks_quick)+"    >>"
    Line Input prompt$,temp$
    If Len(temp$)>0 Then major_ticks_quick=Val(temp$)

    prompt$=" slow chart x-axis max.(sec): "+Str$(slow_chart_scale)+"    >>"
    Line Input prompt$,temp$
    If Len(temp$)>0 Then slow_chart_scale=Val(temp$)

    prompt$ = " minor time ticks (slow - sec.): "+Str$(minor_ticks_slow)+"    >>"
     Line Input prompt$,temp$
    If Len(temp$)>0 Then minor_ticks_slow=Val(temp$)

    prompt$ = " major time ticks (slow-each minor tick): "+Str$(major_ticks_slow)+"    >>"
     Line Input prompt$,temp$
    If Len(temp$)>0 Then major_ticks_slow=Val(temp$)

    prompt$ = " show trace labels (y/n): "+showlabels$+"    >>"
    Line Input prompt$, temp$
    If Len(temp$)>0 Then showLabels$=Left$(temp$,1)

    ' recalc key key values
    CALC_SCREEN_PARAM

    save_chart_config chart_config_file$

End Sub

Sub SAVE_CHART_CONFIG(name$)
  Open name$ For output As #2
  Print #2,chart_mode
  Print #2,max_temp
  Print #2,max_press
  Print #2,refresh_frequency
  Print #2,quick_chart_scale
  Print #2,minor_ticks_quick
  Print #2,major_ticks_quick
  Print #2,slow_chart_scale
  Print #2,minor_ticks_slow
  Print #2,major_ticks_slow
  Print #2,showlabels$
  Close #2
      If MM.Errno<>0 Then
        highlight 1,410,"ERROR SAVING FILE",-1: Pause 2000
     Else
        highlight 1,410, "...SAVED OKAY", -1: Pause 400
    EndIf
End Sub

Sub LOAD_CHART_CONFIG(name$,blModal$)
Open name$ For input As #2
If LCase$(modal$)="true" Then
  If MM.Errno<>0 Then
    highlight 1,410,"ERROR LOADING FILE"+Str$(MM.Errno),-1
    Pause 2000
  EndIf
Else
  Line Input #2,temp$
  chart_mode = Val(temp$)
  Line Input #2,temp$
  max_temp = Val(temp$)
  Line Input #2,temp$
  max_press= Val(temp$)
  Line Input #2,temp$
  refresh_frequency= Val(temp$)
  Line Input #2,temp$
  quick_chart_scale= Val(temp$)
  Line Input #2,temp$
  minor_ticks_quick= Val(temp$)
  Line Input #2,temp$
  major_ticks_quick= Val(temp$)
  Line Input #2,temp$
  slow_chart_scale= Val(temp$)
  Line Input #2,temp$
  minor_ticks_slow= Val(temp$)
  Line Input #2,temp$
  major_ticks_slow= Val(temp$)
  Line Input #2,temp$
  showlabels$=temp$
  If LCase$(blModal$)="true" Then
    If MM.Errno<>0 Then
        highlight 0,0,"ERROR READING CHART CONFIG FILE",-1: Pause 2000
     Else
        highlight 0,0,"CONFIG LOADED OKAY",-1
        Pause 900
    EndIf
  EndIf ' error reading file
EndIf ' modal error display
Close #2
End Sub


Sub DRAW_HORIZONTAL_AXES ' possibly 2 charts to render
Local counter
    If chart_mode = 1 Then offset=0 Else offset=1

    If chart_mode=1 Or chart_mode=3 Then
        Line (origin_x, origin_y-offset*chart_height)-(origin_x+chart_width, origin_y-offset*chart_height),1,"B"
 ' quick axis tick marks
        counter2=0
        For counter = origin_x To (origin_x+chart_width) Step Cint(chart_width/quick_chart_scale*minor_ticks_quick)
            If counter2 Mod major_ticks_quick = 0 Then
              tick_length = 6
 Print @(counter-10,origin_y-chart_height*offset-tick_length+7) counter2*minor_ticks_quick
                Else
             tick_length=3
             EndIf
            Line (counter,origin_y-chart_height*offset+tick_length)-(counter,origin_y-chart_height*offset)
            counter2=counter2+1
        Next counter
    EndIf
' slow chart axis
    If chart_mode =2 Or chart_mode=3 Then
    Line (origin_x, origin_y)-(origin_x+chart_width,origin_y),1
        counter2=0
        For counter = origin_x To (origin_x+chart_width) Step Cint(chart_width/slow_chart_scale*minor_ticks_slow)
           If counter2 Mod major_ticks_slow = 0 Then
              tick_length = 6
Print @(counter-8,origin_y-tick_length+7) counter2*minor_ticks_slow/60
            Else
              tick_length=3
              EndIf
            Line (counter,origin_y+tick_length)-(counter,origin_y)
            counter2=counter2+1
        Next counter
    EndIf

Print @(180,origin_y+12)"time (quick=sec; slow=min)"
End Sub

Sub DRAW_VERTICAL_AXES
Local counter, counter2,counter3
   If chart_mode = 1 Or chart_mode=2 Then
            scale=2
            chart_count=1
        Else
            scale=1
            chart_count=2
    EndIf

    Line (origin_x, origin_y)-(origin_x,origin_y-chart_height*2),1   ' primary axis (pressure)
    Line (origin_x+chart_width,origin_y)-(origin_x+chart_width,origin_y-chart_height*2),1  ' secondary axis ( pressure)

    temp_tick_spacing = Cint(chart_height*scale/max_temp*temp_minor_ticks)
    For counter3 = 0 To chart_count-1
      ' temperature scale
        counter2=0
            For counter = origin_y To origin_y-scale*chart_height Step -temp_tick_spacing
             If counter2 Mod temp_major_ticks = 0 Then
              tick_length = 6
  Print @(origin_x-18,counter-chart_height*counter3+5) counter2*temp_minor_ticks
                  Else
              tick_length=3
            EndIf
                Line (origin_x-tick_length,counter-chart_height*counter3)-(origin_x,counter-chart_height*counter3)
            counter2=counter2+1
        Next counter
        'pressure scale
        counter2=0
      press_tick_spacing = Cint(chart_height*scale/max_press*press_minor_ticks)
         For counter = origin_y To origin_y-scale*chart_height Step -press_tick_spacing
             If counter2 Mod press_major_ticks = 0 Then
  Print @(origin_x+chart_width+3,counter-chart_height*counter3) counter2*press_minor_ticks
        tick_length = 6
          Else
        tick_length=3
       EndIf
             Line (origin_x+chart_width,counter-chart_height*counter3)-(origin_x+chart_width+tick_length,counter-chart_height*counter3),1
            counter2=counter2+1
        Next counter
    Next counter3

print_vert 1,175,"temp ^cv"
print_vert origin_x+chart_width+20,175,"pressure ^psiv"

End Sub

' TABLE PRESENTATION - display readings as table
Sub Tabulate_readings width, height
    Local myX, myY, counter
    myX=110
    myY=140

    If table_exists$="false" Then
      Line (myX-3,myY-15)-(myx+width,myY+height),1,"B"
      highlight myX,myY,"Live Readings",40
      highlight myX, myY+84, "Relay Status (source sensor)",40
      table_exists$="true"
    EndIf
    For counter=1 To 4
    If sensor_status$(counter)="ok" Then
      Print @(myX,myY+counter*12) "Temp Sensor ";counter, Format$(live_readings(counter),"%.2f");" C"
    Else
      Print @(myX,myY+counter*12) Space$(39)
    EndIf
    Next counter
    If sensor_status$(5)="ok" Then
      Print @(myX,myY+60) "Pressure Sensor 1", Format$(live_readings(5),"%.1f");" psi"

    For counter =1 To 3
      If relay_status$(counter)="on" Then Font #1,,1
    Print @(myX,myY+84+counter*12)" Relay ";counter;"(";triggers(counter,1);")",alias$(counter)
      Font #1,1,0
    Next counter

    Print @(myX+45,myY+170) "Elapsed Time = "+ Time$

End Sub


' live readings for chart
Sub SHOW_MONITOR_BAR
Local myCounter
  For mycounter =1 To 5
    If sensor_status$(mycounter)="ok" Then
      Print @(origin_x+(mycounter-1)*55,monitor_y)"S"+Format$(mycounter,"%g")+"="+Format$(live_readings(myCounter),"%4.1f")
    Else
      Print @(origin_x+(mycounter-1)*55,monitor_y)"S"+Format$(mycounter,"%g")+"=n.c"
  EndIf
  Next mycounter
  For mycounter=1 To 3
    If relay_status$(mycounter)="on" Then Font #1,,1
    myName$=alias$(mycounter)
    If myName$="" Then myName$="Relay_"+Format$(mycounter,"%g")
    Print @(origin_x+5*55+(mycounter-1)*49,monitor_y) "["+Left$(myName$,6)+"]"
    Font #1,,0
   Next mycounter

 Print @(2,origin_y+13) "Elapsed Time="+Time$+"  "

End Sub

' LOG Live reading data
Sub LOG_LIVE_CHART_DATA
    Local mycounter
    live_chart_index=live_chart_index+1
    ' if at the end of the buffer loop back
    If live_chart_index = buffer_size+1 Then
        For mycounter=1 To 5
            live_data(1,mycounter)=live_data(live_chart_index-1,mycounter)
        Next mycounter
        live_chart_index=1
    Else
    ' log data
      For mycounter=1 To 5
        live_data(live_chart_index,mycounter)=live_readings(mycounter)
      Next mycounter
    EndIf
  reading_count=reading_count+1
  If reading_count=quick_chart_scale*slow_chart_scale Then reading_count=0
End Sub

Sub UPDATE_SLOW_CHART
Local mycounter, mycolor,x1,x2,y1,y2
If live_chart_index=1 Then
      last_index = buffer_size
    Else
      last_index=live_chart_index-1
    EndIf

If chart_mode =2 Or chart_mode=3 Then
    For mycounter=1 To 5
  If sensor_status$(mycounter)="ok" Then
      x1=slow_chart_x
      x2=x1+slow_dx

      If mycounter<=4 Then
        y1=origin_y-temp_dy*live_data(last_index,mycounter)*scale
        y2=origin_y-temp_dy*live_data(live_chart_index,mycounter)*scale
      Else
        y1=origin_y-press_dy*live_data(last_index,mycounter)*scale
        y2=origin_y-press_dy*live_data(live_chart_index,mycounter)*scale
     EndIf
     If x2<=origin_x+chart_width Then Line (x1,y1)-(x2,y2),1
     If LCase$(Left$(showlabels$,1))="y" Then
      If reading_count Mod Int((50+mycounter*10)/slow_dx) = 0 Then
             Print @(x1,y1) Str$(mycounter)
            EndIf
    EndIf

  EndIf
    Next mycounter
    slow_chart_x=x2
    'Print reading_count*i_period, slow_chart_scale*1000
    If (reading_count*i_period) Mod (slow_chart_scale*1000) = 0 Then
              CLEAR_AREA -1,"true"
              slow_chart_x=origin_x
              CLEAR_AREA origin_x+1,"true"
        EndIf
          If live_chart_index Mod refresh_frequency=0 Then
          CLEAR_AREA -1,"true"
        EndIf
EndIf

' clear the next data points
skip_slow_update:
End Sub

Sub UPDATE_QUICK_CHART
Local mycounter, mycolor, myoffset, x1,x2,x3,x4
If live_chart_index=1 Then
      last_index = buffer_size
    Else
      last_index=live_chart_index-1
    EndIf

If chart_mode=3 Then myoffset=-chart_height Else myoffset=0

If chart_mode =1 Or chart_mode=3 Then
    For mycounter=1 To 5
      If sensor_status$(mycounter)="ok" Then

      x1 = quick_chart_x
      x2=x1+quick_dx
      If mycounter<=4 Then
         y1=origin_y-scale*temp_dy*live_data(last_index,mycounter)+myoffset
         y2=origin_y-scale*temp_dy*live_data(live_chart_index,mycounter)+myoffset
       Else
         y1=origin_y-scale*press_dy*live_data(last_index,mycounter)+myoffset
         y2=origin_y-scale*press_dy*live_data(live_chart_index,mycounter)+myoffset
     EndIf

      If x2<=origin_x+chart_width Then Line (x1,y1)-(x2,y2),1
       If LCase$(Left$(showlabels$,1))="y" Then
          If reading_count Mod Int((40+mycounter*10)/quick_dx) = 0 Then
             Print @(x1,y1) Str$(mycounter)
            EndIf
        'Print x1,y1,x2,y2
        EndIf
    EndIf
    Next mycounter
    quick_chart_x=x2
    If live_chart_index Mod refresh_frequency = 0 Then CLEAR_AREA -1,"false"
    If (reading_count*i_period) Mod (quick_chart_scale*1000) = 0 Then
                  CLEAR_AREA -1,"false"
                  quick_chart_x=origin_x
                  CLEAR_AREA origin_x+1,"false"
            EndIf
' TODO clear the next data points
EndIf
skip_quick_update:
End Sub

' print text vertically on vert axes
Sub PRINT_VERT x,y,text$
Local myCounter,myY
myY=y
  For myCounter = 1 To Len(text$)
    Print @(x,myY+11*myCounter) Mid$(text$,myCounter,1)
  Next myCounter
End Sub


' clear all lines in an area ahead of the new data
Sub CLEAR_AREA x, blSlow$
  Local myX,x1,x2,y1,y2 'area edges
  myX = x ' local copy of x

    If myX=-1 Then
        If blSlow$="true" Then
         myX=slow_chart_x+1
        Else
          myX=quick_chart_x+1
        EndIf
    EndIf

    If myX>=origin_x+chart_width Then myX=origin_x+chart_width-1

    If blSlow$="true" Then
      x2 = myX + refresh_frequency*slow_dx
    Else
      x2 = myX + refresh_frequency*quick_dx
    EndIf

    y1=origin_y-1
    If x2>=origin_x+chart_width Then x2=origin_x+chart_width-1
    y2 = y1-chart_height*scale
    If blSLow$="true" And chart_mode=3 Then y2=y2+12 ' allow space for scale
    If blSlow$="false" And chart_mode=3 Then
          y1=y1-chart_height
          y2=y2-chart_height
          EndIf

    Line(myX,y1)-(x2,y2),0,"BF"

End Sub

'--------- Menu Object -----------------------------------------------------

Sub Main_menu choice$
'' TODO preserve video memory before displaying menu
  Local k$
  Font #1,,1
  table_exists$="false"         ' clear the table exists flag to force redraw
  Line (0,0)-(MM.HRes,40),0,"BF"
If flgMonitoring=0 Then
  highlight 5,0,"[0-Tests]  [1-Triggers]  [2-StartMonitor]  [3-Plot]  [4-Table]  [5-Exit]",-1
Else
  highlight 5,0,"[0-Tests]  [1-Triggers]  [2-EndMonitor]    [3-Plot]  [4-Table]  [5-End]",-1
EndIf
Font #1,,0

k$ = ""
While k$="" Or k$="1" Or k$="3"
  k$ = Inkey$
  k$=LCase$(k$)
  If k$="0" Then choice$="test"
  If k$="1" Then
   Font #1,,1:Print @(75,13)"|_>>[C - Configure] [L - Load] [S-Save] [x-Cancel] "
   Font #1
   EndIf
  If k$="c" Then choice$="triggers.config"
  If k$="l" Then choice$="triggers.load"
  If k$="s" Then choice$="triggers.save"
  If k$="2" And flgMonitoring=1 Then choice$="monitor.end"
  If k$="2" And flgMonitoring=0 Then choice$="monitor.start"
  If k$="3" Then
          Font #1,,1:Print @(200,13)"|_>>[P-Plot][D-Defaults][X-Cancel]"
          Font #1
      EndIf
  If k$="p" Then choice$="plot"
  If k$="d" Then choice$="plot.defaults"
  If k$="4" Then choice$="showtable"
  If k$="5" Then choice$="end"
  If k$="x" Then Line(1,12)-(MM.HRes,40),0,"BF"
Wend

End Sub

' clear menu area
Sub clear_menubar
  Line(0,0)-(MM.HRes,30),0,"BF"
End Sub

Sub highlight x,y,text$,width
  Font #1,,1
  Local myText$, padding
  If width<=0 Then width=80
    padding = width - Len(text$)
    If padding<0 Then padding=0
    myText$ = text$+Space$(padding)
    If x>-1 And y>-1 Then
        Print @(x,y)myText$
      Else
        Print myText$
      EndIf
  Font #1,1
End Sub

Sub clear_screen
  table_exists$="false"
  Cls
End Sub



' -----------------------------------------------------------------
' ------------- MAIN LOOP -----------------------------------------
' -----------------------------------------------------------------

INIT_CONTROLLER   ' configure the pins for all devices

Cls
choice$=""              ' menu choice
Option ERROR CONTINUE     ' leave file error handling to code
load_chart_config chart_config_file$,"false"  ' load default config if exists

myk$ = ""     ' use for keyboard input
While choice$<>"end"
  choice$=""
  main_menu choice$
  If choice$="triggers.config" Then
            MONITOR_END
            clear_screen
            configure_triggers
          EndIf
  If choice$="triggers.load" Then
                                monitor_end
                                load_triggers
                                clear_screen
                              EndIf
  If choice$="triggers.save" Then save_triggers trig_file_name$
  If choice$="test" Then
                MONITOR_END
                test_all
              EndIf
  If choice$="monitor.start" Then
                If LCase$(Left$(resetTimer$,1))="y" Then
                      Time$="00:00:00"
                      resetTimer$=""
                    EndIf
                      MONITOR_START
                EndIf

  If choice$="monitor.end" Then MONITOR_END
  If choice$="plot" Then
          clear_screen
          MONITOR_END
          Print @(1,200) ""
          prompt$ = Space$(25)+"Reset Elapsed Timer (y/n)"
          Line Input prompt$, resetTimer$
          clear_screen
          draw_horizontal_axes
          draw_vertical_axes
          CALC_SCREEN_PARAM
          displayMode$="chart"
          quick_chart_x=origin_x
          slow_chart_x=origin_x
        EndIf
  If choice$="plot.defaults" Then
          MONITOR_END
          clear_screen
          CHART_CONFIG
          save_Chart_config "CH_DEF.txt"
          clear_screen
   EndIf
  If choice$="showtable" Then
            clear_screen
            MONITOR_END
            Print @(1,200) ""
            prompt$ = Space$(25)+"Reset Elapsed Timer (y/n)"
            Line Input prompt$, resetTimer$
            clear_screen
            displayMode$="table"
            MONITOR_START
          EndIf

  ' clear the screen after operation
  If choice$="trigger.save" Or choice$="test" Or choice$="triggers.config" Then
      clear_screen
  EndIf
Wend

MONITOR_END                                         ' turn off monitoring
GoTo Exit_App  'jump the interupt section and exit


'-------------------------------------------------------------
'-------------- Interupts ------------------------------------

' Read temperature and pressure sensors and display on screen
I_readSensors:
    read_temp_All       ' read into live_readings(1)-live_reading(4)
    read_pressure1      ' read into live_readings(5)
    action_all_triggers ' test the trigger levels and swith relays on/off

' DISPLAY results
If DisplayMode$ = "table" Then
' display table
tabulate_readings 240,140
ElseIf DisplayMode$ ="chart" Then
' do charting
    log_live_chart_data
    If chart_mode=1 Or chart_mode=3 Then update_quick_chart
    If chart_mode=2 Or chart_mode=3 Then update_slow_chart
    SHOW_MONITOR_BAR
EndIf

IReturn

EXIT_APP:












